@@ -28,6 +28,7 @@
 #include "core.h"
 #include "debug.h"
 #include "signals.h"
+#include "version.h"
 
 /**************************************************************************
  * Account subsystem signal callbacks
@@ -108,6 +109,28 @@ buddy_signed_off_cb(GaimBuddy *buddy, void *data)
 	gaim_debug_misc("signals test", "buddy-signed-off (%s)\n", buddy->name);
 }
 
+static void
+blist_node_extended_menu_cb(GaimBlistNode *node, void *data)
+{
+	GaimContact *p = (GaimContact *)node;
+	GaimBuddy *b = (GaimBuddy *)node;
+	GaimChat *c = (GaimChat *)node;
+	GaimGroup *g = (GaimGroup *)node;
+
+	if (GAIM_BLIST_NODE_IS_CONTACT(node))
+		gaim_debug_misc("signals test", "blist-node-extended-menu (Contact: %s)\n", p->alias);
+	else if (GAIM_BLIST_NODE_IS_BUDDY(node))
+		gaim_debug_misc("signals test", "blist-node-extended-menu (Buddy: %s)\n", b->name);
+	else if (GAIM_BLIST_NODE_IS_CHAT(node))
+		gaim_debug_misc("signals test", "blist-node-extended-menu (Chat: %s)\n", c->alias);
+	else if (GAIM_BLIST_NODE_IS_GROUP(node))
+		gaim_debug_misc("signals test", "blist-node-extended-menu (Group: %s)\n", g->name);
+	else
+		gaim_debug_misc("signals test", "blist-node-extended-menu (UNKNOWN: %d)\n", node->type);
+
+}
+
+
 /**************************************************************************
  * Connection subsystem signal callbacks
  **************************************************************************/
@@ -153,12 +176,29 @@ displaying_im_msg_cb(GaimAccount *account, GaimConversation *conv,
 }
 
 static void
-displayed_im_msg_cb(GaimConversation *conv, const char *buffer, void *data)
+displayed_im_msg_cb(GaimAccount *account, GaimConversation *conv, const char *buffer, void *data)
 {
 	gaim_debug_misc("signals test", "displayed-im-msg (%s, %s)\n",
 					gaim_conversation_get_name(conv), buffer);
 }
 
+static gboolean
+writing_im_msg_cb(GaimAccount *account, GaimConversation  *conv, char **buffer, void *data)
+{
+	gaim_debug_misc("signals test", "writing-im-msg (%s, %s, %s)\n",
+					gaim_account_get_username(account), gaim_conversation_get_name(conv), *buffer);
+
+	return FALSE;
+
+}
+
+static void
+wrote_im_msg_cb(GaimAccount *account, GaimConversation *conv, const char *buffer, void *data)
+{
+	gaim_debug_misc("signals test", "wrote-im-msg (%s, %s, %s)\n",
+					gaim_account_get_username(account), gaim_conversation_get_name(conv), buffer);
+}
+
 static void
 sending_im_msg_cb(GaimAccount *account, char *recipient, char **buffer, void *data)
 {
@@ -175,16 +215,25 @@ sent_im_msg_cb(GaimAccount *account, const char *recipient, const char *buffer,
 }
 
 static gboolean
-received_im_msg_cb(GaimAccount *account, char **sender, char **buffer,
+receiving_im_msg_cb(GaimAccount *account, char **sender, char **buffer,
 				   int *flags, void *data)
 {
-	gaim_debug_misc("signals test", "received-im-msg (%s, %s, %s, %d)\n",
+	gaim_debug_misc("signals test", "receiving-im-msg (%s, %s, %s, %d)\n",
 					gaim_account_get_username(account), *sender, *buffer,
 					*flags);
 
 	return FALSE;
 }
 
+static void
+received_im_msg_cb(GaimAccount *account, char *sender, char *buffer,
+				   int flags, void *data)
+{
+	gaim_debug_misc("signals test", "received-im-msg (%s, %s, %s, %d)\n",
+					gaim_account_get_username(account), sender, buffer,
+					flags);
+}
+
 static gboolean
 displaying_chat_msg_cb(GaimAccount *account, GaimConversation *conv,
 		       char **buffer, void *data)
@@ -196,12 +245,29 @@ displaying_chat_msg_cb(GaimAccount *account, GaimConversation *conv,
 }
 
 static void
-displayed_chat_msg_cb(GaimConversation *conv, const char *buffer, void *data)
+displayed_chat_msg_cb(GaimAccount *account, GaimConversation *conv, const char *buffer, void *data)
 {
 	gaim_debug_misc("signals test", "displayed-chat-msg (%s, %s)\n",
 					gaim_conversation_get_name(conv), buffer);
 }
 
+static gboolean
+writing_chat_msg_cb(GaimAccount *account, GaimConversation *conv,
+		       char **buffer, void *data)
+{
+	gaim_debug_misc("signals test", "writing-chat-msg (%s, %s)\n",
+					gaim_conversation_get_name(conv), *buffer);
+
+	return FALSE;
+}
+
+static void
+wrote_chat_msg_cb(GaimAccount *account, GaimConversation *conv, const char *buffer, void *data)
+{
+	gaim_debug_misc("signals test", "wrote-chat-msg (%s, %s)\n",
+					gaim_conversation_get_name(conv), buffer);
+}
+
 static gboolean
 sending_chat_msg_cb(GaimAccount *account, char **buffer, int id, void *data)
 {
@@ -219,17 +285,27 @@ sent_chat_msg_cb(GaimAccount *account, const char *buffer, int id, void *data)
 }
 
 static gboolean
-received_chat_msg_cb(GaimAccount *account, char **sender, char **buffer,
+receiving_chat_msg_cb(GaimAccount *account, char **sender, char **buffer,
 					 GaimConversation *chat, void *data)
 {
 	gaim_debug_misc("signals test",
-					"received-chat-msg (%s, %s, %s, %s, %s)\n",
+					"receiving-chat-msg (%s, %s, %s, %s)\n",
 					gaim_account_get_username(account), *sender, *buffer,
 					gaim_conversation_get_name(chat));
 
 	return FALSE;
 }
 
+static void
+received_chat_msg_cb(GaimAccount *account, char *sender, char *buffer,
+					 GaimConversation *chat, void *data)
+{
+	gaim_debug_misc("signals test",
+					"received-chat-msg (%s, %s, %s, %s)\n",
+					gaim_account_get_username(account), sender, buffer,
+					gaim_conversation_get_name(chat));
+}
+
 static void
 conversation_switching_cb(GaimConversation *old_conv,
 						  GaimConversation *new_conv, void *data)
@@ -269,26 +345,40 @@ buddy_typing_cb(GaimConversation *conv, void *data)
 					gaim_conversation_get_name(conv));
 }
 
-static void
-chat_buddy_joining_cb(GaimConversation *conv, const char *user, void *data)
+static gboolean
+chat_buddy_joining_cb(GaimConversation *conv, const char *user,
+					  GaimConvChatBuddyFlags flags, void *data)
 {
-	gaim_debug_misc("signals test", "chat-buddy-joining (%s, %s)\n",
-					gaim_conversation_get_name(conv), user);
+	gaim_debug_misc("signals test", "chat-buddy-joining (%s, %s, %d)\n",
+					gaim_conversation_get_name(conv), user, flags);
+
+	return FALSE;
 }
 
 static void
-chat_buddy_joined_cb(GaimConversation *conv, const char *user, void *data)
+chat_buddy_joined_cb(GaimConversation *conv, const char *user,
+					 GaimConvChatBuddyFlags flags, void *data)
 {
-	gaim_debug_misc("signals test", "chat-buddy-joined (%s, %s)\n",
-					gaim_conversation_get_name(conv), user);
+	gaim_debug_misc("signals test", "chat-buddy-joined (%s, %s, %d)\n",
+					gaim_conversation_get_name(conv), user, flags);
 }
 
 static void
+chat_buddy_flags_cb(GaimConversation *conv, const char *user,
+					GaimConvChatBuddyFlags oldflags, GaimConvChatBuddyFlags newflags, void *data)
+{
+	gaim_debug_misc("signals test", "chat-buddy-flags (%s, %s, %d, %d)\n",
+					gaim_conversation_get_name(conv), user, oldflags, newflags);
+}
+
+static gboolean
 chat_buddy_leaving_cb(GaimConversation *conv, const char *user,
 					  const char *reason, void *data)
 {
 	gaim_debug_misc("signals test", "chat-buddy-leaving (%s, %s, %s)\n",
 					gaim_conversation_get_name(conv), user, reason);
+
+	return FALSE;
 }
 
 static void
@@ -301,10 +391,10 @@ chat_buddy_left_cb(GaimConversation *conv, const char *user,
 
 static void
 chat_inviting_user_cb(GaimConversation *conv, const char *name,
-					  const char *reason, void *data)
+					  char **reason, void *data)
 {
 	gaim_debug_misc("signals test", "chat-inviting-user (%s, %s, %s)\n",
-					gaim_conversation_get_name(conv), name, reason);
+					gaim_conversation_get_name(conv), name, *reason);
 }
 
 static void
@@ -317,7 +407,8 @@ chat_invited_user_cb(GaimConversation *conv, const char *name,
 
 static void
 chat_invited_cb(GaimAccount *account, const char *inviter,
-				const char *room_name, const char *message, void *data)
+				const char *room_name, const char *message,
+				const GHashTable *components, void *data)
 {
 	gaim_debug_misc("signals test", "chat-invited (%s, %s, %s, %s)\n",
 					gaim_account_get_username(account), inviter,
@@ -338,6 +429,15 @@ chat_left_cb(GaimConversation *conv, void *data)
 					gaim_conversation_get_name(conv));
 }
 
+static void
+chat_topic_changed_cb(GaimConversation *conv, const char *who,
+					  const char *topic, void *data)
+{
+	gaim_debug_misc("signals test",
+					"chat-topic-changed (%s topic changed to: \"%s\" by %s)\n",
+					gaim_conversation_get_name(conv), topic,
+					(who) ? who : "unknown");
+}
 /**************************************************************************
  * Core signal callbacks
  **************************************************************************/
@@ -384,6 +484,8 @@ plugin_load(GaimPlugin *plugin)
 						plugin, GAIM_CALLBACK(buddy_signed_on_cb), NULL);
 	gaim_signal_connect(blist_handle, "buddy-signed-off",
 						plugin, GAIM_CALLBACK(buddy_signed_off_cb), NULL);
+	gaim_signal_connect(blist_handle, "blist-node-extended-menu",
+						plugin, GAIM_CALLBACK(blist_node_extended_menu_cb), NULL);
 
 	/* Connection subsystem signals */
 	gaim_signal_connect(conn_handle, "signing-on",
@@ -400,20 +502,32 @@ plugin_load(GaimPlugin *plugin)
 						plugin, GAIM_CALLBACK(displaying_im_msg_cb), NULL);
 	gaim_signal_connect(conv_handle, "displayed-im-msg",
 						plugin, GAIM_CALLBACK(displayed_im_msg_cb), NULL);
+	gaim_signal_connect(conv_handle, "writing-im-msg",
+						plugin, GAIM_CALLBACK(writing_im_msg_cb), NULL);
+	gaim_signal_connect(conv_handle, "wrote-im-msg",
+						plugin, GAIM_CALLBACK(wrote_im_msg_cb), NULL);
 	gaim_signal_connect(conv_handle, "sending-im-msg",
 						plugin, GAIM_CALLBACK(sending_im_msg_cb), NULL);
 	gaim_signal_connect(conv_handle, "sent-im-msg",
 						plugin, GAIM_CALLBACK(sent_im_msg_cb), NULL);
+	gaim_signal_connect(conv_handle, "receiving-im-msg",
+						plugin, GAIM_CALLBACK(receiving_im_msg_cb), NULL);
 	gaim_signal_connect(conv_handle, "received-im-msg",
 						plugin, GAIM_CALLBACK(received_im_msg_cb), NULL);
 	gaim_signal_connect(conv_handle, "displaying-chat-msg",
 						plugin, GAIM_CALLBACK(displaying_chat_msg_cb), NULL);
 	gaim_signal_connect(conv_handle, "displayed-chat-msg",
 						plugin, GAIM_CALLBACK(displayed_chat_msg_cb), NULL);
+	gaim_signal_connect(conv_handle, "writing-chat-msg",
+						plugin, GAIM_CALLBACK(writing_chat_msg_cb), NULL);
+	gaim_signal_connect(conv_handle, "wrote-chat-msg",
+						plugin, GAIM_CALLBACK(wrote_chat_msg_cb), NULL);
 	gaim_signal_connect(conv_handle, "sending-chat-msg",
 						plugin, GAIM_CALLBACK(sending_chat_msg_cb), NULL);
 	gaim_signal_connect(conv_handle, "sent-chat-msg",
 						plugin, GAIM_CALLBACK(sent_chat_msg_cb), NULL);
+	gaim_signal_connect(conv_handle, "receiving-chat-msg",
+						plugin, GAIM_CALLBACK(receiving_chat_msg_cb), NULL);
 	gaim_signal_connect(conv_handle, "received-chat-msg",
 						plugin, GAIM_CALLBACK(received_chat_msg_cb), NULL);
 	gaim_signal_connect(conv_handle, "conversation-switching",
@@ -430,6 +544,8 @@ plugin_load(GaimPlugin *plugin)
 						plugin, GAIM_CALLBACK(chat_buddy_joining_cb), NULL);
 	gaim_signal_connect(conv_handle, "chat-buddy-joined",
 						plugin, GAIM_CALLBACK(chat_buddy_joined_cb), NULL);
+	gaim_signal_connect(conv_handle, "chat-buddy-flags",
+						plugin, GAIM_CALLBACK(chat_buddy_flags_cb), NULL);
 	gaim_signal_connect(conv_handle, "chat-buddy-leaving",
 						plugin, GAIM_CALLBACK(chat_buddy_leaving_cb), NULL);
 	gaim_signal_connect(conv_handle, "chat-buddy-left",
@@ -444,6 +560,8 @@ plugin_load(GaimPlugin *plugin)
 						plugin, GAIM_CALLBACK(chat_joined_cb), NULL);
 	gaim_signal_connect(conv_handle, "chat-left",
 						plugin, GAIM_CALLBACK(chat_left_cb), NULL);
+	gaim_signal_connect(conv_handle, "chat-topic-changed",
+						plugin, GAIM_CALLBACK(chat_topic_changed_cb), NULL);
 
 	/* Core signals */
 	gaim_signal_connect(core_handle, "quitting",
@@ -454,7 +572,9 @@ plugin_load(GaimPlugin *plugin)
 
 static GaimPluginInfo info =
 {
-	2,                                                /**< api_version    */
+	GAIM_PLUGIN_MAGIC,
+	GAIM_MAJOR_VERSION,
+	GAIM_MINOR_VERSION,
 	GAIM_PLUGIN_STANDARD,                             /**< type           */
 	NULL,                                             /**< ui_requirement */
 	0,                                                /**< flags          */
@@ -476,7 +596,9 @@ static GaimPluginInfo info =
 	NULL,                                             /**< destroy        */
 
 	NULL,                                             /**< ui_info        */
-	NULL                                              /**< extra_info     */
+	NULL,                                             /**< extra_info     */
+	NULL,
+	NULL
 };
 
 static void
