@@ -262,6 +262,24 @@ gaim_account_notify_added(GaimAccount *account, const char *id,
 		ui_ops->notify_added(account, remote_user, id, alias, message);
 }
 
+void
+gaim_account_request_add(GaimAccount *account, const char *id,
+                         const char *remote_user, const char *alias,
+                         const char *message)
+{
+	GaimAccountUiOps *ui_ops;
+
+	g_return_if_fail(account     != NULL);
+	g_return_if_fail(remote_user != NULL);
+
+	ui_ops = gaim_accounts_get_ui_ops();
+
+	/*
+	if (ui_ops != NULL && ui_ops->notify_added != NULL)
+		ui_ops->request_add(account, remote_user, id, alias, message);
+		*/
+}
+
 static void
 change_password_cb(GaimAccount *account, GaimRequestFields *fields)
 {
@@ -1339,17 +1357,30 @@ gaim_accounts_write(FILE *fp, GaimAccount *account)
 		if (proxy_type != GAIM_PROXY_USE_GLOBAL &&
 			proxy_type != GAIM_PROXY_NONE &&
 			proxy_type != GAIM_PROXY_USE_ENVVAR) {
+			char *esc;
 			if ((value = gaim_proxy_info_get_host(proxy_info)) != NULL)
-				fprintf(fp, "   <host>%s</host>\n", value);
+			{
+				esc = g_markup_escape_text(value, -1);
+				fprintf(fp, "   <host>%s</host>\n", esc);
+				g_free(esc);
+			}
 
 			if ((int_value = gaim_proxy_info_get_port(proxy_info)) != 0)
 				fprintf(fp, "   <port>%d</port>\n", int_value);
 
 			if ((value = gaim_proxy_info_get_username(proxy_info)) != NULL)
-				fprintf(fp, "   <username>%s</username>\n", value);
+			{
+				esc = g_markup_escape_text(value, -1);
+				fprintf(fp, "   <username>%s</username>\n", esc);
+				g_free(esc);
+			}
 
 			if ((value = gaim_proxy_info_get_password(proxy_info)) != NULL)
-				fprintf(fp, "   <password>%s</password>\n", value);
+			{
+				esc = g_markup_escape_text(value, -1);
+				fprintf(fp, "   <password>%s</password>\n", esc);
+				g_free(esc);
+			}
 		}
 
 		fprintf(fp, "  </proxy>\n");
