@@ -135,7 +135,7 @@ common_send(GaimConversation *conv, const char *message)
 	type = gaim_conversation_get_type(conv);
 	ops  = gaim_conversation_get_ui_ops(conv);
 
-	if (gc->flags & GAIM_CONNECTION_HTML)
+	if (conv->features & GAIM_CONNECTION_HTML)
 		displayed = gaim_markup_linkify(message);
 	else
 		displayed = g_strdup(message);
@@ -185,7 +185,7 @@ common_send(GaimConversation *conv, const char *message)
 		if (sent != NULL && sent[0] != '\0') {
 			GaimMessageFlags msgflags = GAIM_MESSAGE_SEND;
 
-			if (gc && gc->flags & GAIM_CONNECTION_HTML) {
+			if (conv->features & GAIM_CONNECTION_HTML) {
 				err = serv_send_im(gc, gaim_conversation_get_name(conv),
 				                   sent, 0);
 			} else {
@@ -209,7 +209,7 @@ common_send(GaimConversation *conv, const char *message)
 						 gaim_conv_chat_get_id(GAIM_CONV_CHAT(conv)));
 
 		if (sent != NULL && sent[0] != '\0') {
-			if (gc && gc->flags & GAIM_CONNECTION_HTML) {
+			if (conv->features & GAIM_CONNECTION_HTML) {
 				err = serv_chat_send(gc, gaim_conv_chat_get_id(GAIM_CONV_CHAT(conv)), sent);
 			} else {
 				gchar *tmp = gaim_unescape_html(sent);
@@ -224,17 +224,34 @@ common_send(GaimConversation *conv, const char *message)
 	}
 
 	if (err < 0) {
+		const char *who;
+		char *msg;
+
+		who = gaim_conversation_get_name(conv);
+
 		if (err == -E2BIG) {
-			gaim_notify_error(NULL, NULL,
-							  _("Unable to send message. The message is "
-								"too large."), NULL);
+			msg = _("Unable to send message. The message is "
+					"too large.");
+
+			if (!gaim_conv_present_error(who, gaim_connection_get_account(gc), msg)) {
+				msg = g_strdup_printf(_("Unable to send message to %s:"), who);
+				gaim_notify_error(gc, NULL, msg,
+								  _("The message is too large."));
+				g_free(msg);
+			}
 		}
 		else if (err == -ENOTCONN) {
 			gaim_debug(GAIM_DEBUG_ERROR, "conversation",
 					   "Not yet connected.\n");
 		}
 		else {
-			gaim_notify_error(NULL, NULL, _("Unable to send message."), NULL);
+			msg = _("Unable to send message.");
+
+			if (!gaim_conv_present_error(who, gaim_connection_get_account(gc), msg)) {
+				msg = g_strdup_printf(_("Unable to send message to %s:"), who);
+				gaim_notify_error(gc, NULL, msg, NULL);
+				g_free(msg);
+			}
 		}
 	}
 
@@ -730,6 +747,7 @@ gaim_conversation_new(GaimConversationType type, GaimAccount *account,
 					  const char *name)
 {
 	GaimConversation *conv;
+	GaimConnection *gc;
 
 	g_return_val_if_fail(type    != GAIM_CONV_UNKNOWN, NULL);
 	g_return_val_if_fail(account != NULL, NULL);
@@ -749,6 +767,9 @@ gaim_conversation_new(GaimConversationType type, GaimAccount *account,
 		}
 	}
 
+	gc = gaim_account_get_connection(account);
+	g_return_val_if_fail(gc != NULL, NULL);
+
 	conv = g_new0(GaimConversation, 1);
 
 	conv->type         = type;
@@ -762,7 +783,9 @@ gaim_conversation_new(GaimConversationType type, GaimAccount *account,
 	conv->log          = gaim_log_new(type == GAIM_CONV_CHAT ? GAIM_LOG_CHAT :
 									  GAIM_LOG_IM, conv->name, account,
 									  time(NULL));
-
+	/* copy features from the connection. */
+	conv->features = gc->flags;
+	
 
 	if (type == GAIM_CONV_IM)
 	{
@@ -961,7 +984,7 @@ gaim_conversation_destroy(GaimConversation *conv)
 
 		for (node = conv->u.chat->in_room; node != NULL; node = node->next) {
 			if (node->data != NULL)
-				g_free(node->data);
+				gaim_conv_chat_cb_destroy((GaimConvChatBuddy *)node->data);
 			node->data = NULL;
 		}
 
@@ -985,6 +1008,9 @@ gaim_conversation_destroy(GaimConversation *conv)
 			g_free(conv->u.chat->topic);
 		conv->u.chat->topic = NULL;
 
+		if(conv->u.chat->nick)
+			g_free(conv->u.chat->nick);
+
 		g_free(conv->u.chat);
 		conv->u.chat = NULL;
 
@@ -1007,6 +1033,30 @@ gaim_conversation_destroy(GaimConversation *conv)
 	conv = NULL;
 }
 
+
+void
+gaim_conversation_set_features(GaimConversation *conv, GaimConnectionFlags features)
+{
+	GaimConversationUiOps *ops;
+
+	g_return_if_fail(conv != NULL);
+
+	conv->features = features;
+
+	ops = conv->ui_ops;
+	if(ops && ops->updated)
+		ops->updated(conv, GAIM_CONV_UPDATE_FEATURES);	
+}
+
+
+GaimConnectionFlags
+gaim_conversation_get_features(GaimConversation *conv)
+{
+	g_return_val_if_fail(conv != NULL, 0);
+	return conv->features;
+}
+
+
 GaimConversationType
 gaim_conversation_get_type(const GaimConversation *conv)
 {
