@@ -1006,6 +1006,9 @@ add_protocol_options(AccountPrefsDialog *dialog, GtkWidget *parent)
 
 				entry = gtk_entry_new();
 
+				if (gaim_account_option_get_masked(option))
+					gtk_entry_set_visibility(GTK_ENTRY(entry), FALSE);
+
 				if (str_value != NULL)
 					gtk_entry_set_text(GTK_ENTRY(entry), str_value);
 
@@ -2267,6 +2270,33 @@ account_selected_cb(GtkTreeSelection *sel, AccountsWindow *dialog)
 	gtk_widget_set_sensitive(dialog->delete_button, selected);
 }
 
+static gboolean
+account_treeview_double_click_cb(GtkTreeView *treeview, GdkEventButton *event, gpointer user_data)
+{
+	AccountsWindow *dialog;
+	GtkTreePath *path;
+	GtkTreeIter iter;
+	GaimAccount *account;
+
+	dialog = (AccountsWindow *)user_data;
+
+	/* Figure out which node was clicked */
+	if (!gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(dialog->treeview), event->x, event->y, &path, NULL, NULL, NULL))
+		return FALSE;
+	gtk_tree_model_get_iter(GTK_TREE_MODEL(dialog->model), &iter, path);
+	gtk_tree_path_free(path);
+	gtk_tree_model_get(GTK_TREE_MODEL(dialog->model), &iter, COLUMN_DATA, &account, -1);
+
+	if ((account != NULL) && (event->button == 1) &&
+		(event->type == GDK_2BUTTON_PRESS))
+	{
+		gaim_gtk_account_dialog_show(GAIM_GTK_MODIFY_ACCOUNT_DIALOG, account);
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
 static GtkWidget *
 create_accounts_list(AccountsWindow *dialog)
 {
@@ -2299,6 +2329,10 @@ create_accounts_list(AccountsWindow *dialog)
 			gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview)),
 			GTK_SELECTION_MULTIPLE);
 
+	/* Handle double-clicking */
+	g_signal_connect(G_OBJECT(treeview), "button_press_event",
+					 G_CALLBACK(account_treeview_double_click_cb), dialog);
+
 	gtk_container_add(GTK_CONTAINER(sw), treeview);
 	gtk_widget_show(treeview);
 
@@ -2475,6 +2509,28 @@ add_user_cb(GaimGtkAccountAddUserData *data)
 	free_add_user_data(data);
 }
 
+static char *
+make_info(GaimAccount *account, GaimConnection *gc, GaimBuddy *buddy,
+          const char *remote_user, const char *id, const char *alias,
+          const char *msg)
+{
+	return g_strdup_printf(_("%s%s%s%s has made %s his or her buddy%s%s%s"),
+	                       remote_user,
+	                       (alias != NULL ? " ("  : ""),
+	                       (alias != NULL ? alias : ""),
+	                       (alias != NULL ? ")"   : ""),
+	                       (id != NULL
+	                        ? id
+	                        : (gaim_connection_get_display_name(gc) != NULL
+		                         ? gaim_connection_get_display_name(gc)
+		                         : gaim_account_get_username(account))),
+	                       (msg != NULL ? ": " : "."),
+	                       (msg != NULL ? msg  : ""),
+	                       (buddy != NULL
+	                        ? ""
+	                        : _("\n\nDo you wish to add him or her to your buddy list?")));
+}
+
 static void
 gaim_gtk_accounts_notify_added(GaimAccount *account, const char *remote_user,
 							   const char *id, const char *alias,
@@ -2494,21 +2550,7 @@ gaim_gtk_accounts_notify_added(GaimAccount *account, const char *remote_user,
 	data->username = g_strdup(remote_user);
 	data->alias    = (alias != NULL ? g_strdup(alias) : NULL);
 
-	buffer = g_strdup_printf(_("%s%s%s%s has made %s his or her buddy%s%s%s"),
-		remote_user,
-		(alias != NULL ? " ("  : ""),
-		(alias != NULL ? alias : ""),
-		(alias != NULL ? ")"   : ""),
-		(id != NULL
-		 ? id
-		 : (gaim_connection_get_display_name(gc) != NULL
-			? gaim_connection_get_display_name(gc)
-			: gaim_account_get_username(account))),
-		(msg != NULL ? ": " : "."),
-		(msg != NULL ? msg  : ""),
-		(buddy != NULL
-		 ? ""
-		 : _("\n\nDo you wish to add him or her to your buddy list?")));
+	buffer = make_info(account, gc, buddy, remote_user, id, alias, msg);
 
 	if (buddy != NULL)
 	{
@@ -2525,6 +2567,37 @@ gaim_gtk_accounts_notify_added(GaimAccount *account, const char *remote_user,
 	g_free(buffer);
 }
 
+#if 0
+static void
+gaim_gtk_accounts_request_add(GaimAccount *account, const char *remote_user,
+                              const char *id, const char *alias,
+                              const char *msg)
+{
+	char *buffer;
+	GaimConnection *gc;
+	GaimGtkAccountAddUserData *data;
+	GaimBuddy *buddy;
+
+	gc = gaim_account_get_connection(account);
+
+	buddy = gaim_find_buddy(account, remote_user);
+
+	data = g_new0(GaimGtkAccountAddUserData, 1);
+	data->account  = account;
+	data->username = g_strdup(remote_user);
+	data->alias    = (alias != NULL ? g_strdup(alias) : NULL);
+
+	buffer = make_info(account, gc, buddy, remote_user, id, alias, msg);
+
+	gaim_request_action(NULL, NULL, _("Add buddy to your list?"),
+	                    buffer, GAIM_DEFAULT_ACTION_NONE, data, 2,
+	                    _("Add"),    G_CALLBACK(add_user_cb),
+	                    _("Cancel"), G_CALLBACK(free_add_user_data));
+
+	g_free(buffer);
+}
+#endif
+
 static GaimAccountUiOps ui_ops =
 {
 	gaim_gtk_accounts_notify_added
