@@ -30,11 +30,168 @@
 
 #include "userlist.h"
 #include "sync.h"
-
-#define BUDDY_ALIAS_MAXLEN 388
+#include "slplink.h"
 
 static MsnTable *cbs_table;
 
+/**************************************************************************
+ * Main
+ **************************************************************************/
+
+static void
+destroy_cb(MsnServConn *servconn)
+{
+	MsnNotification *notification;
+
+	notification = servconn->cmdproc->data;
+	g_return_if_fail(notification != NULL);
+
+	msn_notification_destroy(notification);
+}
+
+MsnNotification *
+msn_notification_new(MsnSession *session)
+{
+	MsnNotification *notification;
+	MsnServConn *servconn;
+
+	g_return_val_if_fail(session != NULL, NULL);
+
+	notification = g_new0(MsnNotification, 1);
+
+	notification->session = session;
+	notification->servconn = servconn = msn_servconn_new(session, MSN_SERVCONN_NS);
+	msn_servconn_set_destroy_cb(servconn, destroy_cb);
+
+	notification->cmdproc = servconn->cmdproc;
+	notification->cmdproc->data = notification;
+	notification->cmdproc->cbs_table = cbs_table;
+
+	return notification;
+}
+
+void
+msn_notification_destroy(MsnNotification *notification)
+{
+	notification->cmdproc->data = NULL;
+
+	msn_servconn_set_destroy_cb(notification->servconn, NULL);
+
+	msn_servconn_destroy(notification->servconn);
+
+	g_free(notification);
+}
+
+/**************************************************************************
+ * Connect
+ **************************************************************************/
+
+static void
+connect_cb(MsnServConn *servconn)
+{
+	MsnCmdProc *cmdproc;
+	MsnSession *session;
+	GaimAccount *account;
+	char **a, **c, *vers;
+	int i;
+
+	g_return_if_fail(servconn != NULL);
+
+	cmdproc = servconn->cmdproc;
+	session = servconn->session;
+	account = session->account;
+
+	/* Allocate an array for CVR0, NULL, and all the versions */
+	a = c = g_new0(char *, session->protocol_ver - 8 + 3);
+
+	for (i = session->protocol_ver; i >= 8; i--)
+		*c++ = g_strdup_printf("MSNP%d", i);
+
+	*c++ = g_strdup("CVR0");
+
+	vers = g_strjoinv(" ", a);
+
+	if (session->login_step == MSN_LOGIN_STEP_START)
+		msn_session_set_login_step(session, MSN_LOGIN_STEP_HANDSHAKE);
+	else
+		msn_session_set_login_step(session, MSN_LOGIN_STEP_HANDSHAKE2);
+
+	msn_cmdproc_send(cmdproc, "VER", "%s", vers);
+
+	g_strfreev(a);
+	g_free(vers);
+}
+
+gboolean
+msn_notification_connect(MsnNotification *notification, const char *host, int port)
+{
+	MsnServConn *servconn;
+
+	g_return_val_if_fail(notification != NULL, FALSE);
+
+	servconn = notification->servconn;
+
+	msn_servconn_set_connect_cb(servconn, connect_cb);
+	notification->in_use = msn_servconn_connect(servconn, host, port);
+
+	return notification->in_use;
+}
+
+void
+msn_notification_disconnect(MsnNotification *notification)
+{
+	g_return_if_fail(notification != NULL);
+	g_return_if_fail(notification->in_use);
+
+	msn_servconn_disconnect(notification->servconn);
+
+	notification->in_use = FALSE;
+}
+
+/**************************************************************************
+ * Util
+ **************************************************************************/
+
+static void
+group_error_helper(MsnSession *session, const char *msg, int group_id, int error)
+{
+	GaimAccount *account;
+	GaimConnection *gc;
+	char *reason = NULL;
+	char *title = NULL;
+
+	account = session->account;
+	gc = gaim_account_get_connection(account);
+
+	if (error == 224)
+	{
+		if (group_id == 0)
+		{
+			return;
+		}
+		else
+		{
+			const char *group_name;
+			group_name =
+				msn_userlist_find_group_name(session->userlist,
+											 group_id);
+			reason = g_strdup_printf(_("%s is not a valid group."),
+									 group_name);
+		}
+	}
+	else
+	{
+		reason = g_strdup(_("Unknown error."));
+	}
+
+	title = g_strdup_printf(_("%s on %s (%s)"), msg,
+						  gaim_account_get_username(account),
+						  gaim_account_get_protocol_name(account));
+	gaim_notify_error(gc, NULL, title, reason);
+	g_free(title);
+	g_free(reason);
+}
+
 /**************************************************************************
  * Login
  **************************************************************************/
@@ -46,6 +203,8 @@ msn_got_login_params(MsnSession *session, const char *login_params)
 
 	cmdproc = session->notification->cmdproc;
 
+	msn_session_set_login_step(session, MSN_LOGIN_STEP_AUTH_END);
+
 	msn_cmdproc_send(cmdproc, "USR", "TWN S %s", login_params);
 }
 
@@ -60,31 +219,6 @@ cvr_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 					 gaim_account_get_username(account));
 }
 
-static void
-inf_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
-{
-	GaimAccount *account;
-	GaimConnection *gc;
-
-	account = cmdproc->session->account;
-	gc = gaim_account_get_connection(account);
-
-	if (strcmp(cmd->params[1], "MD5"))
-	{
-		msn_cmdproc_show_error(cmdproc, MSN_ERROR_MISC);
-		return;
-	}
-
-	msn_cmdproc_send(cmdproc, "USR", "MD5 I %s",
-					 gaim_account_get_username(account));
-
-	if (cmdproc->error)
-		return;
-
-	gaim_connection_update_progress(gc, _("Requesting to send password"),
-									5, MSN_CONNECT_STEPS);
-}
-
 static void
 usr_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 {
@@ -96,33 +230,22 @@ usr_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 	account = session->account;
 	gc = gaim_account_get_connection(account);
 
-	/*
-	 * We're either getting the passport connect info (if we're on
-	 * MSNP8 or higher), or a challenge request (MSNP7 and lower).
-	 *
-	 * Let's find out.
-	 */
 	if (!g_ascii_strcasecmp(cmd->params[1], "OK"))
 	{
-		const char *friendly = gaim_url_decode(cmd->params[3]);
-
 		/* OK */
+		const char *friendly = gaim_url_decode(cmd->params[3]);
 
 		gaim_connection_set_display_name(gc, friendly);
 
+		msn_session_set_login_step(session, MSN_LOGIN_STEP_SYN);
+
 		msn_cmdproc_send(cmdproc, "SYN", "%s", "0");
-
-		if (cmdproc->error)
-			return;
-
-		gaim_connection_update_progress(gc, _("Retrieving buddy list"),
-										7, MSN_CONNECT_STEPS);
 	}
 	else if (!g_ascii_strcasecmp(cmd->params[1], "TWN"))
 	{
+		/* Passport authentication */
 		char **elems, **cur, **tokens;
 
-		/* Passport authentication */
 		session->nexus = msn_nexus_new(session);
 
 		/* Parse the challenge data. */
@@ -139,40 +262,34 @@ usr_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 
 		g_strfreev(elems);
 
+		msn_session_set_login_step(session, MSN_LOGIN_STEP_AUTH_START);
+
 		msn_nexus_connect(session->nexus);
-
-		gaim_connection_update_progress(gc, _("Password sent"),
-										6, MSN_CONNECT_STEPS);
 	}
-	else if (!g_ascii_strcasecmp(cmd->params[1], "MD5"))
+}
+
+static void
+usr_error(MsnCmdProc *cmdproc, MsnTransaction *trans, int error)
+{
+	MsnErrorType msnerr = 0;
+
+	switch (error)
 	{
-		/* Challenge */
-		const char *challenge;
-		const char *password;
-		char buf[33];
-		md5_state_t st;
-		md5_byte_t di[16];
-		int i;
-
-		challenge = cmd->params[3];
-		password = gaim_account_get_password(account);
-
-		md5_init(&st);
-		md5_append(&st, (const md5_byte_t *)challenge, strlen(challenge));
-		md5_append(&st, (const md5_byte_t *)password, strlen(password));
-		md5_finish(&st, di);
-
-		for (i = 0; i < 16; i++)
-			g_snprintf(buf + (i*2), 3, "%02x", di[i]);
-
-		msn_cmdproc_send(cmdproc, "USR", "MD5 S %s", buf);
-
-		if (cmdproc->error)
+		case 500:
+		case 601:
+		case 910:
+		case 921:
+			msnerr = MSN_ERROR_SERV_UNAVAILABLE;
+			break;
+		case 911:
+			msnerr = MSN_ERROR_AUTH;
+			break;
+		default:
 			return;
-
-		gaim_connection_update_progress(gc, _("Password sent"),
-										6, MSN_CONNECT_STEPS);
+			break;
 	}
+
+	msn_session_set_error(cmdproc->session, msnerr, NULL);
 }
 
 static void
@@ -200,7 +317,8 @@ ver_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 
 	if (!protocol_supported)
 	{
-		msn_cmdproc_show_error(cmdproc, MSN_ERROR_MISC);
+		msn_session_set_error(session, MSN_ERROR_UNSUPPORTED_PROTOCOL,
+							  NULL);
 		return;
 	}
 
@@ -212,18 +330,34 @@ ver_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 /**************************************************************************
  * Log out
  **************************************************************************/
+
 static void
 out_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 {
 	if (!g_ascii_strcasecmp(cmd->params[0], "OTH"))
-		msn_cmdproc_show_error(cmdproc, MSN_ERROR_SIGNOTHER);
+		msn_session_set_error(cmdproc->session, MSN_ERROR_SIGN_OTHER,
+							  NULL);
 	else if (!g_ascii_strcasecmp(cmd->params[0], "SSD"))
-		msn_cmdproc_show_error(cmdproc, MSN_ERROR_SERVDOWN);
+		msn_session_set_error(cmdproc->session, MSN_ERROR_SERV_DOWN, NULL);
+}
+
+void
+msn_notification_close(MsnNotification *notification)
+{
+	g_return_if_fail(notification != NULL);
+
+	if (!notification->in_use)
+		return;
+
+	msn_cmdproc_send_quick(notification->cmdproc, "OUT", NULL, NULL);
+
+	msn_notification_disconnect(notification);
 }
 
 /**************************************************************************
  * Messages
  **************************************************************************/
+
 static void
 msg_cmd_post(MsnCmdProc *cmdproc, MsnCommand *cmd, char *payload,
 			 size_t len)
@@ -235,7 +369,6 @@ msg_cmd_post(MsnCmdProc *cmdproc, MsnCommand *cmd, char *payload,
 	msn_message_parse_payload(msg, payload, len);
 	/* msn_message_show_readable(msg, "Notification", TRUE); */
 
-	msg->remote_user = g_strdup(cmd->params[0]);
 	msn_cmdproc_process_msg(cmdproc, msg);
 
 	msn_message_destroy(msg);
@@ -263,6 +396,7 @@ msg_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 /**************************************************************************
  * Challenges
  **************************************************************************/
+
 static void
 chl_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 {
@@ -286,7 +420,7 @@ chl_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 	for (i = 0; i < 16; i++)
 		g_snprintf(buf + (i*2), 3, "%02x", di[i]);
 
-	trans = msn_transaction_new("QRY", "%s 32", "PROD0038W!61ZTF9");
+	trans = msn_transaction_new(cmdproc, "QRY", "%s 32", "PROD0038W!61ZTF9");
 
 	msn_transaction_set_payload(trans, buf, 32);
 
@@ -296,6 +430,7 @@ chl_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 /**************************************************************************
  * Buddy Lists
  **************************************************************************/
+
 static void
 add_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 {
@@ -331,6 +466,7 @@ add_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 		group_id = -1;
 
 	msn_got_add_user(session, user, list_id, group_id);
+	msn_user_update(user);
 }
 
 static void
@@ -353,15 +489,15 @@ add_error(MsnCmdProc *cmdproc, MsnTransaction *trans, int error)
 	passport = params[1];
 
 	if (!strcmp(list, "FL"))
-		msg = g_strdup_printf("Unable to add user on %s (%s)",
+		msg = g_strdup_printf(_("Unable to add user on %s (%s)"),
 							  gaim_account_get_username(account),
 							  gaim_account_get_protocol_name(account));
 	else if (!strcmp(list, "BL"))
-		msg = g_strdup_printf("Unable to block user on %s (%s)",
+		msg = g_strdup_printf(_("Unable to block user on %s (%s)"),
 							  gaim_account_get_username(account),
 							  gaim_account_get_protocol_name(account));
 	else if (!strcmp(list, "AL"))
-		msg = g_strdup_printf("Unable to permit user on %s (%s)",
+		msg = g_strdup_printf(_("Unable to permit user on %s (%s)"),
 							  gaim_account_get_username(account),
 							  gaim_account_get_protocol_name(account));
 
@@ -369,8 +505,8 @@ add_error(MsnCmdProc *cmdproc, MsnTransaction *trans, int error)
 	{
 		if (error == 210)
 		{
-			reason = g_strdup_printf("%s could not be added because "
-									 "your buddy list is full.", passport);
+			reason = g_strdup_printf(_("%s could not be added because "
+									   "your buddy list is full."), passport);
 		}
 	}
 
@@ -378,12 +514,12 @@ add_error(MsnCmdProc *cmdproc, MsnTransaction *trans, int error)
 	{
 		if (error == 208)
 		{
-			reason = g_strdup_printf("%s is not a valid passport account.",
+			reason = g_strdup_printf(_("%s is not a valid passport account."),
 									 passport);
 		}
 		else
 		{
-			reason = g_strdup_printf("Unknown error.");
+			reason = g_strdup_printf(_("Unknown error."));
 		}
 	}
 
@@ -446,11 +582,18 @@ adg_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 static void
 fln_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 {
-	GaimConnection *gc;
+	MsnSlpLink *slplink;
+	MsnUser *user;
 
-	gc = cmdproc->session->account->gc;
+	user = msn_userlist_find_user(cmdproc->session->userlist, cmd->params[0]);
 
-	serv_got_update(gc, cmd->params[0], FALSE, 0, 0, 0, 0);
+	user->online = FALSE;
+	msn_user_update(user);
+
+	slplink = msn_session_find_slplink(cmdproc->session, cmd->params[0]);
+
+	if (slplink != NULL)
+		msn_slplink_destroy(slplink);
 }
 
 static void
@@ -460,10 +603,7 @@ iln_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 	GaimConnection *gc;
 	MsnUser *user;
 	MsnObject *msnobj;
-	int status = 0;
-	int idle = 0;
 	const char *state, *passport, *friendly;
-	GaimBuddy *b;
 
 	session = cmdproc->session;
 	gc = session->account->gc;
@@ -474,7 +614,6 @@ iln_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 
 	user = msn_userlist_find_user(session->userlist, passport);
 
-	/* serv_got_nick(gc, passport, friendly); */
 	serv_got_alias(gc, passport, friendly);
 
 	msn_user_set_friendly_name(user, friendly);
@@ -485,26 +624,9 @@ iln_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 		msn_user_set_object(user, msnobj);
 	}
 
-	if ((b = gaim_find_buddy(gc->account, passport)) != NULL)
-		status |= ((((b->uc) >> 1) & 0xF0) << 1);
-
-	if (!g_ascii_strcasecmp(state, "BSY"))
-		status |= UC_UNAVAILABLE | (MSN_BUSY << 1);
-	else if (!g_ascii_strcasecmp(state, "IDL"))
-	{
-		status |= UC_UNAVAILABLE | (MSN_IDLE << 1);
-		idle = -1;
-	}
-	else if (!g_ascii_strcasecmp(state, "BRB"))
-		status |= UC_UNAVAILABLE | (MSN_BRB << 1);
-	else if (!g_ascii_strcasecmp(state, "AWY"))
-		status |= UC_UNAVAILABLE | (MSN_AWAY << 1);
-	else if (!g_ascii_strcasecmp(state, "PHN"))
-		status |= UC_UNAVAILABLE | (MSN_PHONE << 1);
-	else if (!g_ascii_strcasecmp(state, "LUN"))
-		status |= UC_UNAVAILABLE | (MSN_LUNCH << 1);
-
-	serv_got_update(gc, passport, TRUE, 0, 0, idle, status);
+	user->online = TRUE;
+	msn_user_set_state(user, state);
+	msn_user_update(user);
 }
 
 static void
@@ -532,8 +654,6 @@ nln_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 	const char *state;
 	const char *passport;
 	const char *friendly;
-	int status = 0;
-	int idle = 0;
 
 	session = cmdproc->session;
 	gc = session->account->gc;
@@ -544,7 +664,6 @@ nln_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 
 	user = msn_userlist_find_user(session->userlist, passport);
 
-	/* serv_got_nick(gc, passport, friendly); */
 	serv_got_alias(gc, passport, friendly);
 
 	msn_user_set_friendly_name(user, friendly);
@@ -563,23 +682,9 @@ nln_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 		}
 	}
 
-	if (!g_ascii_strcasecmp(state, "BSY"))
-		status |= UC_UNAVAILABLE | (MSN_BUSY << 1);
-	else if (!g_ascii_strcasecmp(state, "IDL"))
-	{
-		status |= UC_UNAVAILABLE | (MSN_IDLE << 1);
-		idle = -1;
-	}
-	else if (!g_ascii_strcasecmp(state, "BRB"))
-		status |= UC_UNAVAILABLE | (MSN_BRB << 1);
-	else if (!g_ascii_strcasecmp(state, "AWY"))
-		status |= UC_UNAVAILABLE | (MSN_AWAY << 1);
-	else if (!g_ascii_strcasecmp(state, "PHN"))
-		status |= UC_UNAVAILABLE | (MSN_PHONE << 1);
-	else if (!g_ascii_strcasecmp(state, "LUN"))
-		status |= UC_UNAVAILABLE | (MSN_LUNCH << 1);
-
-	serv_got_update(gc, passport, TRUE, 0, 0, idle, status);
+	user->online = TRUE;
+	msn_user_set_state(user, state);
+	msn_user_update(user);
 }
 
 static void
@@ -628,7 +733,7 @@ not_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 static void
 rea_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 {
-	/* TODO: This might be with us too */
+	/* TODO: This might be for us too */
 
 	MsnSession *session;
 	GaimConnection *gc;
@@ -655,6 +760,21 @@ reg_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 	msn_userlist_rename_group_id(session->userlist, group_id, group_name);
 }
 
+static void
+reg_error(MsnCmdProc *cmdproc, MsnTransaction *trans, int error)
+{
+	int group_id;
+	char **params;
+
+	params = g_strsplit(trans->params, " ", 0);
+
+	group_id = atoi(params[0]);
+
+	group_error_helper(cmdproc->session, _("Unable to rename group"), group_id, error);
+
+	g_strfreev(params);
+}
+
 static void
 rem_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 {
@@ -680,6 +800,7 @@ rem_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 		group_id = -1;
 
 	msn_got_rem_user(session, user, list_id, group_id);
+	msn_user_update(user);
 }
 
 static void
@@ -694,6 +815,21 @@ rmg_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 	msn_userlist_remove_group_id(session->userlist, group_id);
 }
 
+static void
+rmg_error(MsnCmdProc *cmdproc, MsnTransaction *trans, int error)
+{
+	int group_id;
+	char **params;
+
+	params = g_strsplit(trans->params, " ", 0);
+
+	group_id = atoi(params[0]);
+
+	group_error_helper(cmdproc->session, _("Unable to delete group"), group_id, error);
+
+	g_strfreev(params);
+}
+
 static void
 syn_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 {
@@ -701,6 +837,19 @@ syn_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 	int total_users;
 
 	session = cmdproc->session;
+
+	if (cmd->param_count == 2)
+	{
+		/*
+		 * This can happen if we sent a SYN with an up-to-date
+		 * buddy list revision, but we send 0 to get a full list.
+		 * So, error out.
+		 */
+
+		msn_session_set_error(cmdproc->session, MSN_ERROR_BAD_BLIST, NULL);
+		return;
+	}
+
 	total_users  = atoi(cmd->params[2]);
 
 	if (total_users == 0)
@@ -724,6 +873,7 @@ syn_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 /**************************************************************************
  * Misc commands
  **************************************************************************/
+
 static void
 url_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 {
@@ -764,15 +914,15 @@ url_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 
 	if (session->passport_info.file != NULL)
 	{
-		unlink(session->passport_info.file);
+		g_unlink(session->passport_info.file);
 		g_free(session->passport_info.file);
 	}
 
 	if ((fd = gaim_mkstemp(&session->passport_info.file)) == NULL)
 	{
-		gaim_debug(GAIM_DEBUG_ERROR, "msn",
-				   "Error opening temp passport file: %s\n",
-				   strerror(errno));
+		gaim_debug_error("msn",
+						 "Error opening temp passport file: %s\n",
+						 strerror(errno));
 	}
 	else
 	{
@@ -817,9 +967,11 @@ url_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 							 "Error closing temp passport file: %s\n",
 							 strerror(errno));
 
-			unlink(session->passport_info.file);
+			g_unlink(session->passport_info.file);
 			g_free(session->passport_info.file);
+			session->passport_info.file = NULL;
 		}
+#ifdef _WIN32
 		else
 		{
 			/*
@@ -829,10 +981,11 @@ url_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 			char *tmp;
 
 			if ((tmp =
-				 g_strdup_printf("%s.html", session->passport_info.file))
-				!= NULL)
+				g_strdup_printf("%s.html",
+					session->passport_info.file)) != NULL)
 			{
-				if (rename(session->passport_info.file, tmp) == 0)
+				if (g_rename(session->passport_info.file,
+							tmp) == 0)
 				{
 					g_free(session->passport_info.file);
 					session->passport_info.file = tmp;
@@ -841,11 +994,13 @@ url_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 					g_free(tmp);
 			}
 		}
+#endif
 	}
 }
 /**************************************************************************
  * Switchboards
  **************************************************************************/
+
 static void
 rng_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 {
@@ -871,7 +1026,8 @@ rng_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 	swboard->im_user = g_strdup(cmd->params[4]);
 	/* msn_switchboard_add_user(swboard, cmd->params[4]); */
 
-	msn_switchboard_connect(swboard, host, port);
+	if (!msn_switchboard_connect(swboard, host, port))
+		msn_switchboard_destroy(swboard);
 
 	g_free(host);
 }
@@ -884,7 +1040,8 @@ xfr_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 
 	if (strcmp(cmd->params[1], "SB") && strcmp(cmd->params[1], "NS"))
 	{
-		msn_cmdproc_show_error(cmdproc, MSN_ERROR_MISC);
+		/* Maybe we can have a generic bad command error. */
+		gaim_debug_error("msn", "Bad XFR command (%s)\n", cmd->params[1]);
 		return;
 	}
 
@@ -893,19 +1050,6 @@ xfr_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 	if (!strcmp(cmd->params[1], "SB"))
 	{
 		gaim_debug_error("msn", "This shouldn't be handled here.\n");
-#if 0
-		swboard = cmd->trans->data;
-
-		if (swboard != NULL)
-		{
-			msn_switchboard_set_auth_key(swboard, cmd->params[4]);
-
-			if (session->http_method)
-				port = 80;
-
-			msn_switchboard_connect(swboard, host, port);
-		}
-#endif
 	}
 	else if (!strcmp(cmd->params[1], "NS"))
 	{
@@ -913,6 +1057,8 @@ xfr_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 
 		session = cmdproc->session;
 
+		msn_session_set_login_step(session, MSN_LOGIN_STEP_TRANSFER);
+
 		msn_notification_connect(session->notification, host, port);
 	}
 
@@ -922,6 +1068,7 @@ xfr_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 /**************************************************************************
  * Message Types
  **************************************************************************/
+
 static void
 profile_msg(MsnCmdProc *cmdproc, MsnMessage *msg)
 {
@@ -935,19 +1082,42 @@ profile_msg(MsnCmdProc *cmdproc, MsnMessage *msg)
 		return;
 
 	if ((value = msn_message_get_attr(msg, "kv")) != NULL)
+	{
+		if (session->passport_info.kv != NULL)
+			g_free(session->passport_info.kv);
+
 		session->passport_info.kv = g_strdup(value);
+	}
 
 	if ((value = msn_message_get_attr(msg, "sid")) != NULL)
+	{
+		if (session->passport_info.sid != NULL)
+			g_free(session->passport_info.sid);
+
 		session->passport_info.sid = g_strdup(value);
+	}
 
 	if ((value = msn_message_get_attr(msg, "MSPAuth")) != NULL)
+	{
+		if (session->passport_info.mspauth != NULL)
+			g_free(session->passport_info.mspauth);
+
 		session->passport_info.mspauth = g_strdup(value);
+	}
 
 	if ((value = msn_message_get_attr(msg, "ClientIP")) != NULL)
+	{
+		if (session->passport_info.client_ip != NULL)
+			g_free(session->passport_info.client_ip);
+
 		session->passport_info.client_ip = g_strdup(value);
+	}
 
 	if ((value = msn_message_get_attr(msg, "ClientPort")) != NULL)
 		session->passport_info.client_port = ntohs(atoi(value));
+
+	if ((value = msn_message_get_attr(msg, "LoginTime")) != NULL)
+		session->passport_info.sl = atol(value);
 }
 
 static void
@@ -971,7 +1141,7 @@ initial_email_msg(MsnCmdProc *cmdproc, MsnMessage *msg)
 	if (session->passport_info.file == NULL)
 	{
 		MsnTransaction *trans;
-		trans = msn_transaction_new("URL", "%s", "INBOX");
+		trans = msn_transaction_new(cmdproc, "URL", "%s", "INBOX");
 		msn_transaction_queue_cmd(trans, msg->cmd);
 
 		msn_cmdproc_send_trans(cmdproc, trans);
@@ -1024,7 +1194,7 @@ email_msg(MsnCmdProc *cmdproc, MsnMessage *msg)
 	if (session->passport_info.file == NULL)
 	{
 		MsnTransaction *trans;
-		trans = msn_transaction_new("URL", "%s", "INBOX");
+		trans = msn_transaction_new(cmdproc, "URL", "%s", "INBOX");
 		msn_transaction_queue_cmd(trans, msg->cmd);
 
 		msn_cmdproc_send_trans(cmdproc, trans);
@@ -1106,52 +1276,6 @@ system_msg(MsnCmdProc *cmdproc, MsnMessage *msg)
 	g_hash_table_destroy(table);
 }
 
-static void
-connect_cb(MsnServConn *servconn)
-{
-	MsnNotification *notification;
-	MsnCmdProc *cmdproc;
-	MsnSession *session;
-	GaimAccount *account;
-	GaimConnection *gc;
-	char **a, **c, *vers;
-	int i;
-
-	g_return_if_fail(servconn != NULL);
-
-	notification = servconn->data;
-	cmdproc = servconn->cmdproc;
-	session = servconn->session;
-	account = session->account;
-	gc = gaim_account_get_connection(account);
-
-	/* Allocate an array for CVR0, NULL, and all the versions */
-	a = c = g_new0(char *, session->protocol_ver - 8 + 3);
-
-	for (i = session->protocol_ver; i >= 8; i--)
-		*c++ = g_strdup_printf("MSNP%d", i);
-
-	*c++ = g_strdup("CVR0");
-
-	vers = g_strjoinv(" ", a);
-
-	msn_cmdproc_send(cmdproc, "VER", "%s", vers);
-
-	g_strfreev(a);
-	g_free(vers);
-
-	if (cmdproc->error)
-		return;
-
-	session->user = msn_user_new(session->userlist,
-								 gaim_account_get_username(account), NULL);
-
-#if 0
-	gaim_connection_update_progress(gc, _("Syncing with server"),
-									4, MSN_CONNECT_STEPS);
-#endif
-}
-
 void
 msn_notification_add_buddy(MsnNotification *notification, const char *list,
 						   const char *who, const char *store_name,
@@ -1191,6 +1315,10 @@ msn_notification_rem_buddy(MsnNotification *notification, const char *list,
 	}
 }
 
+/**************************************************************************
+ * Init
+ **************************************************************************/
+
 void
 msn_notification_init(void)
 {
@@ -1198,7 +1326,7 @@ msn_notification_init(void)
 
 	cbs_table = msn_table_new();
 
-	/* Syncronous */
+	/* Synchronous */
 	msn_table_add_cmd(cbs_table, "CHG", "CHG", chg_cmd);
 	msn_table_add_cmd(cbs_table, "CHG", "ILN", iln_cmd);
 	msn_table_add_cmd(cbs_table, "ADD", "ADD", add_cmd);
@@ -1208,7 +1336,6 @@ msn_notification_init(void)
 	msn_table_add_cmd(cbs_table, "USR", "XFR", xfr_cmd);
 	msn_table_add_cmd(cbs_table, "SYN", "SYN", syn_cmd);
 	msn_table_add_cmd(cbs_table, "CVR", "CVR", cvr_cmd);
-	msn_table_add_cmd(cbs_table, "INF", "INF", inf_cmd);
 	msn_table_add_cmd(cbs_table, "VER", "VER", ver_cmd);
 	msn_table_add_cmd(cbs_table, "REA", "REA", rea_cmd);
 	/* msn_table_add_cmd(cbs_table, "PRP", "PRP", prp_cmd); */
@@ -1219,7 +1346,7 @@ msn_notification_init(void)
 	msn_table_add_cmd(cbs_table, "RMG", "RMG", rmg_cmd);
 	msn_table_add_cmd(cbs_table, "XFR", "XFR", xfr_cmd);
 
-	/* Asyncronous */
+	/* Asynchronous */
 	msn_table_add_cmd(cbs_table, NULL, "IPG", ipg_cmd);
 	msn_table_add_cmd(cbs_table, NULL, "MSG", msg_cmd);
 	msn_table_add_cmd(cbs_table, NULL, "NOT", not_cmd);
@@ -1241,7 +1368,10 @@ msn_notification_init(void)
 	msn_table_add_cmd(cbs_table, "fallback", "XFR", xfr_cmd);
 
 	msn_table_add_error(cbs_table, "ADD", add_error);
+	msn_table_add_error(cbs_table, "REG", reg_error);
+	msn_table_add_error(cbs_table, "RMG", rmg_error);
 	/* msn_table_add_error(cbs_table, "REA", rea_error); */
+	msn_table_add_error(cbs_table, "USR", usr_error);
 
 	msn_table_add_msg_type(cbs_table,
 						   "text/x-msmsgsprofile",
@@ -1262,57 +1392,3 @@ msn_notification_end(void)
 {
 	msn_table_destroy(cbs_table);
 }
-
-MsnNotification *
-msn_notification_new(MsnSession *session)
-{
-	MsnNotification *notification;
-	MsnServConn *servconn;
-
-	g_return_val_if_fail(session != NULL, NULL);
-
-	notification = g_new0(MsnNotification, 1);
-
-	notification->session = session;
-	notification->servconn = servconn = msn_servconn_new(session, MSN_SERVER_NS);
-	notification->cmdproc = servconn->cmdproc;
-	msn_servconn_set_connect_cb(servconn, connect_cb);
-
-	if (session->http_method)
-		servconn->http_data->server_type = "NS";
-
-	servconn->cmdproc->cbs_table = cbs_table;
-
-	return notification;
-}
-
-void
-msn_notification_destroy(MsnNotification *notification)
-{
-	msn_servconn_destroy(notification->servconn);
-
-	g_free(notification);
-}
-
-gboolean
-msn_notification_connect(MsnNotification *notification, const char *host, int port)
-{
-	MsnServConn *servconn;
-
-	g_return_val_if_fail(notification != NULL, FALSE);
-
-	servconn = notification->servconn;
-
-	return (notification->in_use = msn_servconn_connect(servconn, host, port));
-}
-
-void
-msn_notification_disconnect(MsnNotification *notification)
-{
-	g_return_if_fail(notification != NULL);
-
-	notification->in_use = FALSE;
-
-	if (notification->servconn->connected)
-		msn_servconn_disconnect(notification->servconn);
-}
