@@ -1469,7 +1469,7 @@ silc_connected(SilcClient client, SilcClientConnection conn,
 	case SILC_CLIENT_CONN_SUCCESS_RESUME:
 		gaim_connection_set_state(gc, GAIM_CONNECTED);
 		serv_finish_login(gc);
-		unlink(silcgaim_session_file(gaim_account_get_username(sg->account)));
+		g_unlink(silcgaim_session_file(gaim_account_get_username(sg->account)));
 
 		/* Send any UMODEs configured for account */
 		reject_watch = gaim_account_get_bool(sg->account, "reject-watch", FALSE);
@@ -1489,7 +1489,7 @@ silc_connected(SilcClient client, SilcClientConnection conn,
 		break;
 	case SILC_CLIENT_CONN_ERROR:
 		gaim_connection_error(gc, _("Error during connecting to SILC Server"));
-		unlink(silcgaim_session_file(gaim_account_get_username(sg->account)));
+		g_unlink(silcgaim_session_file(gaim_account_get_username(sg->account)));
 		break;
 
 	case SILC_CLIENT_CONN_ERROR_KE:
@@ -1504,7 +1504,7 @@ silc_connected(SilcClient client, SilcClientConnection conn,
 		gaim_connection_error(gc,
 				      _("Resuming detached session failed. "
 					"Press Reconnect to create new connection."));
-		unlink(silcgaim_session_file(gaim_account_get_username(sg->account)));
+		g_unlink(silcgaim_session_file(gaim_account_get_username(sg->account)));
 		break;
 
 	case SILC_CLIENT_CONN_ERROR_TIMEOUT:
@@ -1531,7 +1531,7 @@ silc_disconnected(SilcClient client, SilcClientConnection conn,
 	SilcGaim sg = gc->proto_data;
 
 	if (sg->resuming && !sg->detaching)
-		unlink(silcgaim_session_file(gaim_account_get_username(sg->account)));
+		g_unlink(silcgaim_session_file(gaim_account_get_username(sg->account)));
 
 	sg->conn = NULL;
 
@@ -1603,15 +1603,15 @@ silc_get_auth_method(SilcClient client, SilcClientConnection conn,
 
 	/* Check configuration if we have this connection configured.  If we
 	   have then return that data immediately, as it's faster way. */
+	if (gaim_account_get_bool(sg->account, "pubkey-auth", FALSE)) {
+		completion(TRUE, SILC_AUTH_PUBLIC_KEY, NULL, 0, context);
+		return;
+	}
 	if (gc->account->password && *gc->account->password) {
 		completion(TRUE, SILC_AUTH_PASSWORD, gc->account->password,
 			   strlen(gc->account->password), context);
 		return;
 	}
-	if (gaim_account_get_bool(sg->account, "pubkey-auth", FALSE)) {
-		completion(TRUE, SILC_AUTH_PUBLIC_KEY, NULL, 0, context);
-		return;
-	}
 
 	/* Resolve the authentication method from server, as we may not know it. */
 	internal = silc_calloc(1, sizeof(*internal));
@@ -1822,7 +1822,7 @@ silc_detach(SilcClient client, SilcClientConnection conn,
 
 	/* Save the detachment data to file. */
 	file = silcgaim_session_file(gaim_account_get_username(sg->account));
-	unlink(file);
+	g_unlink(file);
 	silc_file_writefile(file, detach_data, detach_data_len);
 }
 
