@@ -101,7 +101,7 @@ struct _parse_tree {
 
 parse_tree null_parse_tree = { 
 	"", 
-	{}, 
+	{NULL}, 
 	0,
 };
 
@@ -681,7 +681,7 @@ static void message_failed(GaimConnection *gc, ZNotice_t notice, struct sockaddr
 	}
 }
 
-static void handle_message(GaimConnection *gc,ZNotice_t notice, struct sockaddr_in from)
+static void handle_message(GaimConnection *gc,ZNotice_t notice)
 {
 	zephyr_account* zephyr = gc->proto_data;
 
@@ -1054,7 +1054,6 @@ static gint check_notify_tzc(gpointer data)
 	GaimConnection *gc = (GaimConnection *)data;
 	zephyr_account* zephyr = gc->proto_data;
 	parse_tree *newparsetree = read_from_tzc(zephyr);
-	struct sockaddr_in from;
 	if (newparsetree != NULL) {
 		gchar *spewtype;
 		if ( (spewtype =  tree_child(find_node(newparsetree,"tzcspew"),2)->contents) ) {
@@ -1078,7 +1077,7 @@ static gint check_notify_tzc(gpointer data)
 				notice.z_default_format = "Class $class, Instance $instance:\n" "To: @bold($recipient) at $time $date\n" "From: @bold($1) <$sender>\n\n$2";
 				notice.z_message_len = strlen(msg) + 3;
 				notice.z_message = buf;
-				handle_message(gc,notice,from);
+				handle_message(gc, notice);
 				g_free(msg);
 				/*				  g_free(zsig); */
 				g_free(buf);
@@ -1166,7 +1165,7 @@ static gint check_notify_zeph02(gpointer data)
 		case UNSAFE:
 		case UNACKED:
 		case ACKED:
-			handle_message(gc,notice, from);
+			handle_message(gc,notice);
 			break;
 		case SERVACK:
 			if (!(g_ascii_strcasecmp(notice.z_message, ZSRVACK_NOTSENT))) {
@@ -1316,7 +1315,7 @@ static void process_zsubs(zephyr_account *zephyr)
 	gchar buff[BUFSIZ];
 
 	fname = g_strdup_printf("%s/.zephyr.subs", gaim_home_dir());
-	f = fopen(fname, "r");
+	f = g_fopen(fname, "r");
 	if (f) {
 		char **triple;
 		char *recip;
@@ -1412,7 +1411,7 @@ static void process_anyone(GaimConnection *gc)
 	}
 
 	filename = g_strconcat(gaim_home_dir(), "/.anyone", NULL);
-	if ((fd = fopen(filename, "r")) != NULL) {
+	if ((fd = g_fopen(filename, "r")) != NULL) {
 		while (fgets(buff, BUFSIZ, fd)) {
 			strip_comments(buff);
 			if (buff[0]) {
@@ -1717,7 +1716,7 @@ static void write_zsubs(zephyr_account *zephyr)
 	char **triple;
 
 	fname = g_strdup_printf("%s/.zephyr.subs", gaim_home_dir());
-	fd = fopen(fname, "w");
+	fd = g_fopen(fname, "w");
 
 	if (!fd) {
 		g_free(fname);
@@ -1779,7 +1778,7 @@ static void write_anyone(GaimConnection *gc)
 	FILE *fd;
 	zephyr_account* zephyr = gc->proto_data;
 	fname = g_strdup_printf("%s/.anyone", gaim_home_dir());
-	fd = fopen(fname, "w");
+	fd = g_fopen(fname, "w");
 	if (!fd) {
 		g_free(fname);
 		return;
@@ -2321,12 +2320,13 @@ static int zephyr_send_typing(GaimConnection *gc, const char *who, int typing) {
 		gaim_debug_info("zephyr", "who is null\n");
 		recipient = local_zephyr_normalize(zephyr,"");
 	} else {
+		char *comma = strrchr(who, ',');
 		/* Don't ping broadcast (chat) recipients */
 		/* The strrchr case finds a realm-stripped broadcast subscription
 		   e.g. comma is the last character in the string */
-		if ((strrchr(who, ',') && ((strrchr(who, ',') == (who+strlen(who)-1)) || ((char *)(strrchr(who, ',')[1] == '@')))))
+		if (comma && ( (*(comma+1) == '\0') || (*(comma+1) == '@')))
 			return 0;
-		
+
 		recipient = local_zephyr_normalize(zephyr,who);
 	}
 
