@@ -1460,6 +1460,8 @@ void serv_got_chat_invite(GaimConnection *gc, const char *name,
 	GaimAccount *account;
 	char buf2[BUF_LONG];
 	struct chat_invite_data *cid = g_new0(struct chat_invite_data, 1);
+	char *name_escaped = g_markup_escape_text(name, -1);
+	char *who_escaped = g_markup_escape_text(who, -1);
 
 	account = gaim_connection_get_account(gc);
 
@@ -1468,16 +1470,19 @@ void serv_got_chat_invite(GaimConnection *gc, const char *name,
 
 	if (message != NULL)
 	{
-		char *escaped = g_markup_escape_text(message, -1);
+		char *message_escaped = g_markup_escape_text(message, -1);
 		g_snprintf(buf2, sizeof(buf2),
 				   _("%s has invited %s to the chat room %s:\n<b>%s</b>"),
-				   who, gaim_account_get_username(account), name, escaped);
-		g_free(escaped);
+				   who_escaped, gaim_account_get_username(account), name_escaped, message_escaped);
+		g_free(message_escaped);
 	}
 	else
 		g_snprintf(buf2, sizeof(buf2),
 				   _("%s has invited %s to the chat room %s\n"),
-				   who, gaim_account_get_username(account), name);
+				   who_escaped, gaim_account_get_username(account), name_escaped);
+
+	g_free(name_escaped);
+	g_free(who_escaped);
 
 	cid->gc = gc;
 	cid->components = data;
